#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
from halcomp import halcore
from lcncdefs import AXISCHAR
import joint

"""
page_joint_setup:
	select/deselect joints

halcomp:
	get username & halname from owner

halfile generation

inifile generation

"""


class Joint_Setup:
	def __init__(self, builder, widgets):
		# List of widgets to update on joint tab change.
		# These must all have a set_joint() method.
		self._widgets = widgets

		# Create joint notebook and tabs
		self.nbk = gtk.Notebook()
		self.selections = []
		for i in range(9):
			vbx_dummy = gtk.VBox();
			self.nbk.append_page(vbx_dummy, gtk.Label("Joint %d (\"%s\")" % (i, AXISCHAR[i].upper())))
			self.selections.append([0] * 5)

		frm = builder.get_object("frm_header")
		frm.set_label(None)
		self._lbl_header = builder.get_object("lbl_header")

		# The same page/widgets are reused for all joints
		self._gtk_joint_setup = builder.get_object("page_joint_setup")

		# Show page_joint_setup for joint 0
		self._jointno = -1;
		self.nbk.connect("switch-page", self.nbk_switch_page)
		self.nbk.set_current_page(0)

	def nbk_switch_page(self, notebook, page, page_num):
		# Move widgets to current tab
		if self._jointno > -1:
			page = notebook.get_nth_page(self._jointno)
			page.remove(self._gtk_joint_setup)
		self._jointno = page_num;
		for widget in self._widgets:
			widget.set_jointno(self._jointno)
		self._lbl_header.set_text("Joint %d setup" % self._jointno)
		page = notebook.get_nth_page(self._jointno)
		page.pack_start(self._gtk_joint_setup)
		page.show_all()


class Joint_cbx_wrapper(object):
	def __init__(self, cbx, noseltext, termtype):
		self._value = [0]*9
		self._jointno = 0
		self._cbx = cbx
		self._store = gtk.ListStore(str, object)
		self._renderer = gtk.CellRendererText()
		self._cbx.set_model(self._store)
		self._cbx.pack_start(self._renderer)
		self._cbx.add_attribute(self._renderer, "text", 0)
		self._noseltext = noseltext
		self._termtype = termtype

	# Save value for previous joint and display current
	def set_jointno(self, jointno):
		# At startup, the indices can be -1; guard against this
		if self._cbx.get_active() > -1:
			self._value[self._jointno] = self._cbx.get_active()
		self._jointno = jointno
		self._cbx.set_active(self._value[self._jointno])

	# Get available components from platform
	def update(self, platform):
		self._store.clear()
		self._store.append([self._noseltext, None])
		for hw in platform.get_connected():
			for terminal in hw.get_terminals(self._termtype):
				self._store.append([terminal.get_name(), terminal.get_halcomp()])
		self._cbx.set_active(0)

	# Get the selected component for joint number 'jointno'
	def get_comp(self, jointno):
		index = self._value[jointno]
		return self._store[index][1]
