#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from hwic import hwic

class Hardware(object):
	UPCONN = None

	def __init__(self, name):
		self._name = name
		self._hwic = []
		self._terminals = []

	# HWC: list of Hardware class types
	# result: [[card, [hwic, hwic, ...]], ...], where
	#	'card' is one of the cards that may connect to self
	#	'hwic' is the connector on which the card can connect
	def get_connectable(self, HWC):
		result = []
		for hwc in HWC:
			conn_list = []
			for hwic in self._hwic:
				if (hwic.is_connected()):
					continue
				if isinstance(hwic, hwc.UPCONN):
					conn_list.append(hwic)
			if len(conn_list) > 0:
				pair = [hwc]
				pair.append(conn_list)
				result.append(pair)
		return result

	def get_connected(self):
		connected = []
		for hwic in self._hwic:
			if hwic.is_connected():
				connected.append(hwic.get_connected())
		return connected

	def get_terminals(self, termtype):
		terminals = []
		for terminal in self._terminals:
			if type(terminal) == termtype:
				terminals.append(terminal)
		for hwic in self._hwic:
			if hwic.is_connected():
				for terminal in hwic.get_connected().get_terminals(termtype):
					terminals.append(terminal)
		return terminals

	def get_usercomps(self, compclass):
		usercomps = []
		for halcomp in self._halcomps:
			if isinstance(halcomp, compclass):
				usercomps.append(halcomp)
		for hwic in self._hwic:
			if hwic.is_connected():
				#usercomps.extend(hwic.get_connected().get_usercomps(compclass)
				for usercomp in hwic.get_connected().get_usercomps(compclass):
					usercomps.append(usercomp)
		return usercomps

	def __repr__(self):
		return self._name


# "Terminals" are what the user hooks up when configuring
class Terminal(object):
	def __init__(self, name, halcomp):
		self._name = name
		self._halcomp = halcomp

	def get_halcomp(self):
		return self._halcomp

	def get_name(self):
		return self._name


class TermControl(Terminal):
	def __init__(self, name, halcomp):
		Terminal.__init__(self, name, halcomp)


class TermFeedback(Terminal):
	def __init__(self, name, halcomp):
		Terminal.__init__(self, name, halcomp)


class TermOutput(Terminal):
	def __init__(self, name, halcomp):
		Terminal.__init__(self, name, halcomp)


class TermInput(Terminal):
	def __init__(self, name, halcomp):
		Terminal.__init__(self, name, halcomp)


### The root hw that other hw is added to
class Platform(Hardware):
	def __init__(self):
		Hardware.__init__(self, "Computer")
		self._hwic.append(hwic.Direct(self, "N/A"))

	# Return
	def get_card_index(self, card):
		index = 0
		for acard in self.get_connected():
			if card == acard:
				return index
			if type(card) == type(acard):
				index += 1
		raise Exception("'card' not found in platform")


class EPP(Hardware):
	UPCONN = hwic.Direct

	def __init__(self):
		Hardware.__init__(self, "EPP")
